@@ -30,6 +30,7 @@ import uk.ac.starlink.ttools.func.Gaia;
 import uk.ac.starlink.ttools.func.KCorrections;
 import uk.ac.starlink.ttools.func.Lists;
 import uk.ac.starlink.ttools.func.Maths;
+import uk.ac.starlink.ttools.func.Shapes;
 import uk.ac.starlink.ttools.func.Strings;
 import uk.ac.starlink.ttools.func.Tilings;
 import uk.ac.starlink.ttools.func.Times;
@@ -135,6 +136,7 @@ public class JELUtils {
                 KCorrections.class,
                 Lists.class,
                 Maths.class,
+                Shapes.class,
                 Strings.class,
                 Tilings.class,
                 Times.class,
