@@ -7,7 +7,7 @@ import uk.ac.starlink.ttools.plot2.DataGeom;
 import uk.ac.starlink.ttools.plot2.PlotType;
 import uk.ac.starlink.ttools.plot2.Plotter;
 import uk.ac.starlink.ttools.plot2.SurfaceFactory;
-import uk.ac.starlink.ttools.plot2.config.ConfigKey;
+import uk.ac.starlink.ttools.plot2.config.PerUnitConfigKey;
 import uk.ac.starlink.ttools.plot2.config.StyleKeys;
 import uk.ac.starlink.ttools.plot2.data.Coord;
 import uk.ac.starlink.ttools.plot2.data.FloatingCoord;
@@ -26,7 +26,6 @@ import uk.ac.starlink.ttools.plot2.layer.LinearFitPlotter;
 import uk.ac.starlink.ttools.plot2.layer.LabelPlotter;
 import uk.ac.starlink.ttools.plot2.layer.MarkForm;
 import uk.ac.starlink.ttools.plot2.layer.MultiPointForm;
-import uk.ac.starlink.ttools.plot2.layer.Normalisation;
 import uk.ac.starlink.ttools.plot2.layer.PairLinkForm;
 import uk.ac.starlink.ttools.plot2.layer.PlaneCorrelationCoordSet;
 import uk.ac.starlink.ttools.plot2.layer.PlaneEllipseCoordSet;
@@ -37,6 +36,7 @@ import uk.ac.starlink.ttools.plot2.layer.ShapePlotter;
 import uk.ac.starlink.ttools.plot2.layer.SizeXyForm;
 import uk.ac.starlink.ttools.plot2.layer.Stats1Plotter;
 import uk.ac.starlink.ttools.plot2.layer.TracePlotter;
+import uk.ac.starlink.ttools.plot2.layer.Unit;
 import uk.ac.starlink.ttools.plot2.paper.PaperTypeSelector;
 
 /**
@@ -127,21 +127,21 @@ public class PlanePlotType implements PlotType {
         Plotter[] shapePlotters =
             ShapePlotter.createShapePlotters( forms, ShapeMode.MODES_2D );
         list.addAll( Arrays.asList( shapePlotters ) );
-        ConfigKey<Normalisation> normKey = StyleKeys.NORMALISE;
+        PerUnitConfigKey<Unit> unitKey = null;
         list.addAll( Arrays.asList( new Plotter[] {
             new LinePlotter(),
             new LinearFitPlotter( true ),
             new LabelPlotter(),
-            new ContourPlotter(),
+            new ContourPlotter( true ),
             new GridPlotter( true ),
             new FillPlotter( true ),
             new TracePlotter( true ),
-            new HistogramPlotter( PlaneDataGeom.X_COORD, true, normKey ),
+            new HistogramPlotter( PlaneDataGeom.X_COORD, true, unitKey ),
             new FixedKernelDensityPlotter( PlaneDataGeom.X_COORD, true,
-                                           normKey ),
-            new KnnKernelDensityPlotter( PlaneDataGeom.X_COORD, true, normKey ),
+                                           unitKey ),
+            new KnnKernelDensityPlotter( PlaneDataGeom.X_COORD, true, unitKey ),
             new DensogramPlotter( PlaneDataGeom.X_COORD, true ),
-            new Stats1Plotter( PlaneDataGeom.X_COORD, true, normKey ),
+            new Stats1Plotter( PlaneDataGeom.X_COORD, true, unitKey ),
             FunctionPlotter.PLANE,
         } ) );
         return list.toArray( new Plotter[ 0 ] );
