@@ -43,6 +43,7 @@ import uk.ac.starlink.ttools.plot2.config.ConfigMeta;
 import uk.ac.starlink.ttools.plot2.config.DoubleConfigKey;
 import uk.ac.starlink.ttools.plot2.config.OptionConfigKey;
 import uk.ac.starlink.ttools.plot2.config.RampKeySet;
+import uk.ac.starlink.ttools.plot2.config.SliderSpecifier;
 import uk.ac.starlink.ttools.plot2.config.StyleKeys;
 import uk.ac.starlink.ttools.plot2.data.Coord;
 import uk.ac.starlink.ttools.plot2.data.CoordGroup;
@@ -298,7 +299,9 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
             "that value multiplied by the bin width.",
             "</p>",
         } );
-        return DoubleConfigKey.createSliderKey( meta, 0, 0, 1, false );
+        return DoubleConfigKey
+              .createSliderKey( meta, 0, 0, 1, false, false,
+                                SliderSpecifier.TextOption.ENTER_ECHO );
     }
 
     /**
