@@ -41,6 +41,8 @@ import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
 import uk.ac.starlink.ttools.plot2.config.ConfigMeta;
 import uk.ac.starlink.ttools.plot2.config.IntegerConfigKey;
+import uk.ac.starlink.ttools.plot2.config.OptionConfigKey;
+import uk.ac.starlink.ttools.plot2.config.PerUnitConfigKey;
 import uk.ac.starlink.ttools.plot2.config.RampKeySet;
 import uk.ac.starlink.ttools.plot2.config.StyleKeys;
 import uk.ac.starlink.ttools.plot2.data.Coord;
@@ -72,6 +74,10 @@ public class SkyDensityPlotter
     private final FloatingCoord weightCoord_;
     private final boolean reportAuxKeys_;
 
+    /** Weighting coordinate. */
+    private static final FloatingCoord WEIGHT_COORD =
+        FloatingCoord.WEIGHT_COORD;
+
     /** Report key for HEALPix tile area in square degrees. */
     public static final ReportKey<Double> TILESIZE_REPKEY =
         ReportKey.createDoubleKey( new ReportMeta( "tile_sqdeg",
@@ -82,7 +88,7 @@ public class SkyDensityPlotter
         ReportKey
        .createIntegerKey( new ReportMeta( "abs_level",
                                           "Absolute HEALPix Level" ),
-                                    false );
+                          false );
     private static final ReportKey<Integer> RELLEVEL_REPKEY =
         ReportKey
        .createIntegerKey( new ReportMeta( "rel_level",
@@ -93,8 +99,7 @@ public class SkyDensityPlotter
                                   true );
 
     private static final AuxScale SCALE = AuxScale.COLOR;
-    private static final FloatingCoord WEIGHT_COORD =
-        FloatingCoord.WEIGHT_COORD;
+
     private static final RampKeySet RAMP_KEYS = StyleKeys.AUX_RAMP;
     private static final ConfigKey<Integer> LEVEL_KEY =
         IntegerConfigKey.createSpinnerPairKey(
@@ -118,9 +123,71 @@ public class SkyDensityPlotter
                 "</p>",
             } )
         , -3, 29, -8, "Abs", "Rel", ABSLEVEL_REPKEY, RELLEVEL_REPKEY );
-    private static final ConfigKey<Double> TRANSPARENCY_KEY =
+
+    /** Config key for transparency. */
+    public static final ConfigKey<Double> TRANSPARENCY_KEY =
         StyleKeys.TRANSPARENCY;
 
+    /** Config key for combination mode. */
+    public static final ConfigKey<Combiner> COMBINER_KEY;
+
+    /** Config key for solid angle units. */
+    public static final PerUnitConfigKey<SolidAngleUnit> ANGLE_KEY;
+
+    static {
+        ConfigMeta angleMeta = new ConfigMeta( "perunit", "Per Unit" );
+        ConfigMeta combineMeta = new ConfigMeta( "combine", "Combine" );
+        angleMeta.setShortDescription( "Solid angle unit for densities" );
+        angleMeta.setXmlDescription( new String[] {
+            "<p>Defines the unit of sky area used for scaling density-like",
+            "combinations",
+            "(e.g. <code>" + combineMeta.getShortName() + "</code>=" +
+            "<code>" + Combiner.DENSITY + "</code> or",
+            "<code>" + Combiner.WEIGHTED_DENSITY + "</code>).",
+            "If the Combination mode is calculating values per unit area,",
+            "this configures the area scale in question.", 
+            "For non-density-like combination modes",
+            "(e.g. <code>" + combineMeta.getShortName() + "</code>=" +
+            "<code>" + Combiner.SUM + "</code> or ",
+            "<code>" + Combiner.MEAN + "</code>)",
+            "it has no effect.",
+            "</p>", 
+        } );
+        combineMeta.setShortDescription( "Value combination mode" );
+        combineMeta.setXmlDescription( new String[] {
+            "<p>Defines how values contributing to the same density map bin",
+            "are combined together to produce the value assigned to that bin,",
+            "and hence its colour.",
+            "The combined values are the weights, but if the",
+            "<code>" + WEIGHT_COORD.getInput().getMeta().getShortName()
+                     + "</code>",
+            "is left blank, a weighting of unity is assumed.",
+            "</p>",
+            "<p>For density-like values",
+            "(<code>" + Combiner.DENSITY + "</code>,",
+            "<code>" + Combiner.WEIGHTED_DENSITY + "</code>)",
+            "the scaling is additionally influenced by the",
+            "<code>" + angleMeta.getShortName() + "</code> parameter.",
+            "</p>",
+        } );
+        ANGLE_KEY =
+            new PerUnitConfigKey<SolidAngleUnit>(
+                    angleMeta, SolidAngleUnit.class,
+                    SolidAngleUnit.getKnownUnits(),
+                    SolidAngleUnit.DEGREE2 );
+        COMBINER_KEY =
+                new OptionConfigKey<Combiner>( combineMeta, Combiner.class,
+                                               Combiner.getKnownCombiners(),
+                                               Combiner.WEIGHTED_DENSITY ) {
+            public String getXmlDescription( Combiner combiner ) {
+                return combiner.getDescription();
+            }
+        }.setOptionUsage()
+         .addOptionsXml();
+    }
+    private static final ReportKey<Combiner.Type> CTYPE_REPKEY =
+        ANGLE_KEY.getCombinerTypeReportKey();
+
     /**
      * Constructor.
      *
@@ -187,7 +254,8 @@ public class SkyDensityPlotter
         List<ConfigKey> keyList = new ArrayList<ConfigKey>();
         keyList.add( LEVEL_KEY );
         if ( weightCoord_ != null ) {
-            keyList.add( StyleKeys.COMBINER );
+            keyList.add( COMBINER_KEY );
+            keyList.add( ANGLE_KEY );
         }
         if ( reportAuxKeys_ ) {
             keyList.addAll( Arrays.asList( RAMP_KEYS.getKeys() ) );
@@ -206,8 +274,9 @@ public class SkyDensityPlotter
         Shader shader = Shaders.fade( ramp.getShader(), scaleAlpha );
         Combiner combiner = weightCoord_ == null
                           ? Combiner.COUNT
-                          : config.get( StyleKeys.COMBINER );
-        return new SkyDenseStyle( level, scaling, shader, combiner );
+                          : config.get( COMBINER_KEY );
+        SolidAngleUnit unit = config.get( ANGLE_KEY );
+        return new SkyDenseStyle( level, scaling, shader, combiner, unit );
     }
 
     public PlotLayer createLayer( DataGeom geom, DataSpec dataSpec,
@@ -266,6 +335,7 @@ public class SkyDensityPlotter
         private final Scaling scaling_;
         private final Shader shader_;
         private final Combiner combiner_;
+        private final SolidAngleUnit unit_;
 
         /**
          * Constructor.
@@ -279,13 +349,15 @@ public class SkyDensityPlotter
          *                    colour map entries
          * @param   shader   colour map
          * @param   combiner  value combination mode for bin calculation
+         * @param   unit     solid angle unit for density combinations
          */
         public SkyDenseStyle( int level, Scaling scaling, Shader shader,
-                              Combiner combiner ) {
+                              Combiner combiner, SolidAngleUnit unit ) {
             level_ = level;
             scaling_ = scaling;
             shader_ = shader;
             combiner_ = combiner;
+            unit_ = unit;
         }
 
         /**
@@ -311,6 +383,7 @@ public class SkyDensityPlotter
             code = 23 * code + scaling_.hashCode();
             code = 23 * code + shader_.hashCode();
             code = 23 * code + combiner_.hashCode();
+            code = 23 * code + unit_.hashCode();
             return code;
         }
 
@@ -321,7 +394,8 @@ public class SkyDensityPlotter
                 return this.level_ == other.level_
                     && this.scaling_.equals( other.scaling_ )
                     && this.shader_.equals( other.shader_ )
-                    && this.combiner_.equals( other.combiner_ );
+                    && this.combiner_.equals( other.combiner_ )
+                    && this.unit_.equals( other.unit_ );
             }
             else {
                 return false;
@@ -372,6 +446,9 @@ public class SkyDensityPlotter
                 public int getCoordIndex() {
                     return icWeight_;
                 }
+                public ValueInfo getAxisInfo( DataSpec dataSpec ) {
+                    return getCombinedInfo( dataSpec );
+                }
                 public void adjustAuxRange( Surface surface, DataSpec dataSpec,
                                             DataStore dataStore,
                                             Object[] knownPlans,
@@ -388,7 +465,8 @@ public class SkyDensityPlotter
                     SkyDensityPlan splan =
                         getSkyPlan( knownPlans, level, dataSpec );
                     if ( splan != null ) {
-                        splan.extendVisibleRange( range, ssurf );
+                        splan.extendVisibleRange( range, ssurf,
+                                                  getBinFactor( level ) );
                     }
                     else {
                         BinList.Result binResult =
@@ -410,9 +488,10 @@ public class SkyDensityPlotter
          * @return  tile renderer
          */
         private SkyTileRenderer createTileRenderer( SkySurface surface ) {
+            int level = getLevel( surface );
             return SkyTileRenderer
-                  .createRenderer( surface, Rotation.IDENTITY,
-                                   getLevel( surface ) );
+                  .createRenderer( surface, Rotation.IDENTITY, level,
+                                   getBinFactor( level ) );
         }
 
         /**
@@ -430,14 +509,17 @@ public class SkyDensityPlotter
         }
 
         /**
-         * Constructs an object which can map sky positions to a pixel
-         * index in a HEALPix grid.
+         * Returns the value by which all bin result values should be
+         * multiplied for a given level.
          *
-         * @param   surface  target plot surface
-         * @return   sky pixer
+         * @param   level  HEALPix level for plotting
+         * @return   bin multiplication factor
          */
-        private SkyPixer createSkyPixer( SkySurface surface ) {
-            return new SkyPixer( getLevel( surface ) );
+        private double getBinFactor( int level ) {
+            double binExtent = Tilings.healpixSqdeg( level )
+                             / dstyle_.unit_.getExtentInSquareDegrees();
+            Combiner.Type ctype = dstyle_.combiner_.getType();
+            return ctype.getBinFactor( binExtent );
         }
 
         /**
@@ -452,7 +534,8 @@ public class SkyDensityPlotter
          */
         private BinList readBins( SkySurface surface, DataSpec dataSpec,
                                   DataStore dataStore ) {
-            SkyPixer skyPixer = createSkyPixer( surface );
+            int level = getLevel( surface );
+            SkyPixer skyPixer = new SkyPixer( level );
             BinList binList = null;
             long npix = skyPixer.getPixelCount();
             Combiner combiner = dstyle_.combiner_;
@@ -515,6 +598,29 @@ public class SkyDensityPlotter
             return null;
         }
 
+       /**
+         * Returns the metadata for the combined values.
+         *
+         * @param  dataSpec  data specification
+         * @return   metadata for gridded cells
+         */
+        private ValueInfo getCombinedInfo( DataSpec dataSpec ) {
+            final ValueInfo weightInfo;
+            if ( icWeight_ < 0 || dataSpec.isCoordBlank( icWeight_ ) ) {
+                weightInfo = new DefaultValueInfo( "1", Double.class,
+                                                   "Weight unspecified"
+                                                 + ", taken as unity" ); 
+            }
+            else {
+                ValueInfo[] winfos = dataSpec.getUserCoordInfos( icWeight_ );
+                weightInfo = winfos != null && winfos.length == 1
+                           ? winfos[ 0 ]
+                           : new DefaultValueInfo( "Weight", Double.class );
+            }
+            return dstyle_.combiner_
+                  .createCombinedInfo( weightInfo, dstyle_.unit_ );
+        }
+
         /**
          * Drawing implementation for the sky density map.
          */
@@ -599,6 +705,7 @@ public class SkyDensityPlotter
                     map.put( RELLEVEL_REPKEY, new Integer( relLevel ) );
                     map.put( TILESIZE_REPKEY, new Double( tileSize ) );
                     map.put( HPXTABLE_REPKEY, createExportTable( splan ) );
+                    map.put( CTYPE_REPKEY, splan.combiner_.getType() );
                 }
                 return map;
             }
@@ -637,22 +744,10 @@ public class SkyDensityPlotter
 
             /* Construct a column containing bin contents, given that the
              * bin index is row number. */
-            ValueInfo weightInfo;
-            if ( icWeight_ < 0 || dataSpec.isCoordBlank( icWeight_ ) ) {
-                weightInfo = null;
-            }
-            else {
-                ValueInfo[] winfos = dataSpec.getUserCoordInfos( icWeight_ );
-                weightInfo = winfos != null && winfos.length == 1
-                           ? winfos[ 0 ]
-                           : null;
-            }
-            if ( weightInfo == null ) {
-                weightInfo = new DefaultValueInfo( "data", Double.class );
-            }
-            ValueInfo dataInfo = combiner.createCombinedInfo( weightInfo );
+            ValueInfo dataInfo = getCombinedInfo( dataSpec );
             ColumnData dataCol =
-                BinResultColumnData.createInstance( dataInfo, binResult );
+                BinResultColumnData.createInstance( dataInfo, binResult,
+                                                    getBinFactor( level ) );
 
             /* Combine these into a table and return. */
             final long nrow = skyPixer.getPixelCount();
@@ -728,8 +823,10 @@ public class SkyDensityPlotter
          *
          * @param  range     range to submit values to
          * @param  surface   viewing surface
+         * @param  binFactor   factor for multiplying bin result values
          */
-        public void extendVisibleRange( Range range, SkySurface surface ) {
+        public void extendVisibleRange( Range range, SkySurface surface,
+                                        double binFactor ) {
             Rectangle bounds = surface.getPlotBounds();
             int nx = bounds.width;
             int ny = bounds.height;
@@ -745,6 +842,7 @@ public class SkyDensityPlotter
                 double[] dpos = surface.graphicsToData( point, null );
                 if ( dpos != null ) {
                     double dval =
+                        binFactor *
                         binResult_.getBinValue( skyPixer.getIndex( dpos ) );
                     range.submit( dval );
                 }
