@@ -490,6 +490,10 @@ public class FuncTest extends TestCase {
         assertEquals( Maths.E, Maths.exp( 1.0 ), TINY );
         assertEquals( 1.0, Maths.ln( Maths.E ), TINY );
         assertEquals( 2.0, Maths.log10( 100.0 ), TINY );
+        assertEquals( 1000, Maths.exp10( 3 ), TINY );
+        assertEquals( 0.001, Maths.exp10( -3 ), TINY );
+        assertEquals( Maths.PI, Maths.exp10( Maths.log10( Maths.PI ) ), TINY );
+        assertEquals( Maths.E, Maths.log10( Maths.exp10( Maths.E ) ), TINY );
 
         assertEquals( 256.0, Maths.pow( 2, 8 ) );
 
@@ -590,6 +594,9 @@ public class FuncTest extends TestCase {
         assertEquals( "some text", Strings.trim( "  some text  " ) );
         assertEquals( "some text", Strings.trim( "some text" ) );
 
+        assertEquals( "RR%20Lyr", Strings.urlEncode( "RR Lyr" ) );
+        assertEquals( "RR Lyr", Strings.urlDecode( "RR%20Lyr" ) );
+
         // These examples from sec 3.5.1 of IAU-endorsed document
         // "Specifications concerning designations" at
         // http://cds.u-strasbg.fr/vizier/Dic/iau-spec.htx
@@ -762,6 +769,40 @@ public class FuncTest extends TestCase {
         assertEquals( "-90:00:00.000", CoordsDegrees.degreesToDms( -90, 3 ) );
     }
 
+    public void testShapes() {
+        assertTrue( Shapes.isInside(0.5,0.5, 0,0, 0,1, 1,1, 1,0) );
+        assertFalse( Shapes.isInside(0,0, Arrays.array(10,20, 20,20, 20,10)) );
+
+        assertEquals( 23, Shapes.polyLine( -100, new double[] { -5,23 } ) );
+        assertEquals( 23, Shapes.polyLine( +100, new double[] { -5,23 } ) );
+        assertEquals( 23, Shapes.polyLine( -5, new double[] { -5,23 } ) );
+
+        assertEquals( 1, Shapes.polyLine( 1, new double[] { 0,0, 2,2, } ) );
+        assertEquals( -1, Shapes.polyLine( -1, new double[] { Math.PI,Math.PI,
+                                                              -1e9,-1e9 } ) );
+        assertEquals( -2000, Shapes.polyLine( 1000, -5,10., 5,-10 ) );
+
+        double[] line = new double[] { 0,0, 1,1, 3,-1, 10,-1 };
+        for ( int i = 0; i < line.length / 2; i++ ) {
+            assertEquals( line[ 2 * i + 1 ],
+                          Shapes.polyLine( line[ 2 * i + 0 ], line ) );
+        }
+        assertEquals( -5, Shapes.polyLine( -5, line ) );
+        assertEquals( 0.1, Shapes.polyLine( 0.1, line ) );
+        assertEquals( 0.99, Shapes.polyLine( 0.99, line ) );
+        assertEquals( 0, Shapes.polyLine( 2, line ) );
+        assertEquals( -1, Shapes.polyLine( 4, line ) );
+        assertEquals( -1, Shapes.polyLine( 1e100, line ) );
+
+        double[] tri = new double[] { -1,-1, 1,1, -1,2, };
+        assertTrue( Shapes.isInside( -0.1, 0, tri ) );
+        assertTrue( !Shapes.isInside( +0.1, 0, tri ) );
+        assertTrue( Shapes.isInside( 0, 1, tri ) );
+        assertTrue( !Shapes.isInside( 0, 2, tri ) );
+        assertTrue( !Shapes.isInside( 0, -1, tri ) );
+        assertTrue( !Shapes.isInside( 1, 0, tri ) );
+    }
+
     public void testTilings() {
         final double pi4 = 4.0 * Math.PI;
         assertEquals( pi4, Tilings.healpixSteradians( 0 ) * 12 );
